#include <stdint.h>
#include <avr/io.h>
#include <stdio.h>
#include <util/delay.h>


#include "clock.h"
#include "stepper.h"
#include "validLed.h"

#define STEPPER_REG_1_DIR DDRC
#define STEPPER_REG_1 PORTC

#define HOUR_MOTOR_DIRECTION PC0
#define HOUR_MOTOR_PULSE PC1
#define MINUTE_MOTOR_DIRECTION PC2
#define MINUTE_MOTOR_PULSE PC3

#define STEPPER_CTRL_RESET PC4

#define STEPPER_REG_2_DIR DDRA
#define STEPPER_REG_2 PORTA

#define HOUR_MOTOR_ENABLE PA0
#define MINUTE_MOTOR_ENABLE PA1

#define STEPPER_REG_3_DIR DDRB
#define STEPPER_REG_3 PORTB

#define HOUR_POSITION_SWITCH PB1
#define MINUTE_POSITION_SWITCH PB2


uint8_t stepperEnableFlag = 0;


static inline void hourMotorEnable() {
  STEPPER_REG_2 |= (1 << HOUR_MOTOR_ENABLE);
}

static inline void hourMotorDisable() {
  STEPPER_REG_2 &= ~(1 << HOUR_MOTOR_ENABLE);
}

static inline void minuteMotorEnable() {
  STEPPER_REG_2 |= (1 << MINUTE_MOTOR_ENABLE);
}

static inline void minuteMotorDisable() {
  STEPPER_REG_2 &= ~(1 << MINUTE_MOTOR_ENABLE);
}


static inline void hourMotorBackward() {
  STEPPER_REG_1 &= ~(1 << HOUR_MOTOR_DIRECTION);
}

static inline void hourMotorForward() {
  STEPPER_REG_1 |= (1 << HOUR_MOTOR_DIRECTION);
}

static inline void minuteMotorForward() {
  STEPPER_REG_1 &= ~(1 << MINUTE_MOTOR_DIRECTION);
}

static inline void minuteMotorBackward() {
  STEPPER_REG_1 |= (1 << MINUTE_MOTOR_DIRECTION);
}


static inline void hourMotorPulse() {
  STEPPER_REG_1 |= (1 << HOUR_MOTOR_PULSE);
  _delay_us(50);
  STEPPER_REG_1 &= ~(1 << HOUR_MOTOR_PULSE);
}

static inline void minuteMotorPulse() {
  STEPPER_REG_1 |= (1 << MINUTE_MOTOR_PULSE);
  _delay_us(50);
  STEPPER_REG_1 &= ~(1 << MINUTE_MOTOR_PULSE);
}


static inline void stepperCtrlReset() {
  STEPPER_REG_1 &= ~(1 << STEPPER_CTRL_RESET);
  _delay_ms(1);
  STEPPER_REG_1 |= (1 << STEPPER_CTRL_RESET);
}

void stepperInit() {
  // stepper
  STEPPER_REG_1_DIR |= (1 << STEPPER_CTRL_RESET) |
      (1 << MINUTE_MOTOR_PULSE) | (1 << MINUTE_MOTOR_DIRECTION) |
      (1 << HOUR_MOTOR_PULSE) | (1 << HOUR_MOTOR_DIRECTION);
  STEPPER_REG_2_DIR |= (1 << MINUTE_MOTOR_ENABLE) | (1 << HOUR_MOTOR_ENABLE);

  STEPPER_REG_1 &= ~(1 << MINUTE_MOTOR_PULSE);
  STEPPER_REG_1 &= ~(1 << MINUTE_MOTOR_DIRECTION);
  STEPPER_REG_1 &= ~(1 << HOUR_MOTOR_PULSE);
  STEPPER_REG_1 &= ~(1 << HOUR_MOTOR_DIRECTION);

  STEPPER_REG_2 &= ~(1 << MINUTE_MOTOR_ENABLE);
  STEPPER_REG_2 &= ~(1 << HOUR_MOTOR_ENABLE);

  // position switches
  STEPPER_REG_3_DIR &= ~(1 << MINUTE_POSITION_SWITCH);
  STEPPER_REG_3_DIR &= ~(1 << HOUR_POSITION_SWITCH);

  // pull-up on
  STEPPER_REG_3 |= (1 << MINUTE_POSITION_SWITCH) | (1 << HOUR_POSITION_SWITCH);


  stepperCtrlReset();
}




void stepperEnable() {
  stepperEnableFlag = 1;

}
void stepperEngine() {

  static uint8_t stepperState = 100;

  static uint16_t currentMinutePosition = 0;
  static uint16_t minuteStepPosition = 0;
  static int8_t minuteDirection = 0;

  static uint16_t currentHourPosition = 0;
  static uint16_t hourStepPosition = 0;
  static int8_t hourDirection = 0;


  if (clockNextStep()) {

    static uint8_t oldPositionSwitch = 255;
    uint8_t positionSwitch = PINB & ((1 << PB1) | (1 << PB2));
    uint8_t minutePositionSwitch = (positionSwitch & (1 << PB2)) != 0;
    uint8_t hourPositionSwitch = (positionSwitch & (1 << PB1)) != 0;
    if (oldPositionSwitch != positionSwitch) {
      oldPositionSwitch = positionSwitch;
      printf("POS: %02d\n", positionSwitch);
      printf("MinuteSwitch: %d\n", minutePositionSwitch);
      printf("HourSwitch:   %d\n", hourPositionSwitch);

    }


    clock_t clock = clockGetClock();

    switch (stepperState) {
    case 100:
      printf("100 Minute zeroing ");
      minuteMotorEnable();
      minuteMotorBackward();
      currentMinutePosition = 0;
      stepperState = 101;
      break;
    case 101:
      printf("101 ");
      currentMinutePosition++;
      printf("%d\n", currentMinutePosition);
      minuteMotorPulse();
      if (minutePositionSwitch == 0) {
        stepperState = 102;
      }
      if (currentMinutePosition > 500) {
        stepperState = 200;
      }
      break;
    case 102:
      printf("102 Minute zeroed\n");
      currentMinutePosition = 0;
      minuteMotorDisable();
      stepperState = 103;
      break;
    case 103:
      printf("103 Hour zeroing ");
      hourMotorEnable();
      hourMotorBackward();
      currentHourPosition = 0;
      stepperState = 104;
      break;
    case 104:
      printf("104 ");
      currentHourPosition++;
      printf("%d\n", currentHourPosition);
      hourMotorPulse();
      if (hourPositionSwitch == 0) {
        stepperState = 105;
      }
      if (currentHourPosition > 500) {
        stepperState = 201;
      }
      break;
    case 105:
      printf("105 Hour zeroed\n");
      currentHourPosition = 0;
      hourMotorDisable();
      stepperState = 0;
      break;
    case 200:
      printf("200 Minute zeroing error\n");
      {
        static uint8_t ledToggle = 0;
        if (ledToggle != 0) {
          ledToggle = 0;
          validLedDisable();
        } else {
          ledToggle = 1;
          validLedEnable();
        }
      }
      break;
    case 201:
      printf("201 Hour zeroing error\n");
      {
        static uint8_t ledToggle = 0;
        if (ledToggle != 0) {
          ledToggle = 0;
          validLedDisable();
        } else {
          ledToggle = 1;
          validLedEnable();
        }
      }
      break;
    case 0:
      //printf("0 ");
      if (stepperEnableFlag == 1) {
        // is current minute position different from desired one?
        minuteStepPosition = clock.minute * STEPS_PER_MINUTE;

        if (currentMinutePosition != minuteStepPosition) {
          printf("M:\n");

          stepperState = 1;
        }
      }
      break;
    case 1:
      printf("1 ");
      // switch on the minute motor and set the direction
      minuteMotorEnable();

      if (currentMinutePosition > minuteStepPosition) {
        printf("b\n");
        minuteDirection = -1;
        minuteMotorBackward();
      } else {
        printf("f\n");
        minuteDirection = 1;
        minuteMotorForward();
      }

      stepperState = 2;
      break;
    case 2:
      printf("2 ");

      // move one step
      currentMinutePosition += minuteDirection;

      minuteMotorPulse();

      printf("d: %d, c: %d\n", minuteStepPosition, currentMinutePosition);

      if (currentMinutePosition == minuteStepPosition) {
        stepperState = 3;
      }
      break;
    case 3:
      printf("3\n");

      // switch off the minute motor
      minuteMotorDisable();

      stepperState = 4;
      break;
    case 4:
      printf("4 ");

      // is current hour position different from desired one?
      hourStepPosition = clock.hour * STEPS_PER_HOUR;

      if (currentHourPosition != hourStepPosition) {
        printf(" H:\n");

        stepperState = 5;
      } else {
        // done
        printf("\n");
        stepperState = 0;
      }
      break;
    case 5:
      printf("5 ");

      // switch on the hour motor and set the direction
      hourMotorEnable();

      if (currentHourPosition > hourStepPosition) {
        printf("b\n");
        hourDirection = -1;
        hourMotorBackward();
      } else {
        printf("f\n");
        hourDirection = 1;
        hourMotorForward();
      }

      stepperState = 6;
      break;
    case 6:
      printf("6 ");

      // move one step
      currentHourPosition += hourDirection;

      hourMotorPulse();

      printf("d: %d, c: %d\n", hourStepPosition, currentHourPosition);

      if (currentHourPosition == hourStepPosition) {
        stepperState = 7;
      }
      break;
    case 7:
      printf("7\n");

      // switch off the hour motor
      hourMotorDisable();

      stepperState = 8;
      break;
    case 8:
      printf("8\n");

      // done
      stepperState = 0;
      break;
    default:
      printf("default\n");
      stepperState = 0;
      break;
    }
  }
}

